package pps.core.prediction.service;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import pps.core.common.constant.BusinessConstant;
import pps.core.prediction.entity.ConfigShowEnt;
import pps.core.prediction.entity.PlantPredictedPowerShortTermEnt;
import pps.core.prediction.mapper.ConfigShowMapper;
import pps.core.prediction.mapper.PlantPredictedPowerShortTermMapper;
import pps.core.prediction.service.data.plant_predicted_power_mid_term.GetPlantPredictedPowerMidTermOutput;
import pps.core.prediction.service.data.plant_predicted_power_short_term.GetPlantPredictedPowerShortTermInput;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.base.util.XDateUtils;
import xstartup.base.util.XStringUtils;
import xstartup.data.XListResult;
import xstartup.feature.api.annotation.XApiGet;

import java.util.Date;
import java.util.List;


@XService
@XText("电站预测功率短期72小时数据")
public class PlantPredictedPowerShortTermService {

    @XApiGet
    @XText("根据电站预测3天的预测数据")
    public XListResult<GetPlantPredictedPowerMidTermOutput> getThreePlantPredictedPowerShortTermList(XContext context, GetPlantPredictedPowerShortTermInput input) {
        PlantPredictedPowerShortTermMapper mapper = context.getBean(PlantPredictedPowerShortTermMapper.class);
        QueryWrapper<PlantPredictedPowerShortTermEnt> queryWrapper = setQueryWrapper(input.getPlantId(), input.getStartTime(), 3);
        List<PlantPredictedPowerShortTermEnt> list = mapper.selectList(queryWrapper);
        List<GetPlantPredictedPowerMidTermOutput> outputs = XCopyUtils.copyNewList(list, GetPlantPredictedPowerMidTermOutput.class);
        ThirdActivePowerServiceHelper.getActivePower(context, outputs, input.getPlantId(), null);
        return XListResult.success(outputs);
    }

    @XApiGet
    @XText("根据电站预测1天的预测数据")
    public XListResult<GetPlantPredictedPowerMidTermOutput> getOnePlantPredictedPowerShortTermList(XContext context, GetPlantPredictedPowerShortTermInput input) {
        String day = input.getStartTime();
        //判断是否需要启用演示配置
        ConfigShowMapper configMapper = context.getBean(ConfigShowMapper.class);
        ConfigShowEnt configShowEnt = configMapper.selectOne(new LambdaQueryWrapper<ConfigShowEnt>()
                .last("LIMIT 1"));
        Date showTime = null;
        if (ObjectUtil.isAllNotEmpty(configShowEnt, configShowEnt.getChoiceDate(), configShowEnt.getChoiceDate())) {
            //只要当天才优化
            if (DateUtil.parse(input.getStartTime()).compareTo(DateUtil.beginOfDay(DateUtil.date())) == 0) {
                day = configShowEnt.getIsEnable().equals(0) ? DateUtil.date(configShowEnt.getChoiceDate()).toString(BusinessConstant.DATE_FORMAT_DAY) : day;
                showTime = DateUtil.parse(day + " " + DateUtil.date().toString(BusinessConstant.TIME_FORMAT));
            }
        }
        PlantPredictedPowerShortTermMapper mapper = context.getBean(PlantPredictedPowerShortTermMapper.class);
        QueryWrapper<PlantPredictedPowerShortTermEnt> queryWrapper = setQueryWrapper(input.getPlantId(), day, 1);
        List<PlantPredictedPowerShortTermEnt> list = mapper.selectList(queryWrapper);
        List<GetPlantPredictedPowerMidTermOutput> outputs = XCopyUtils.copyNewList(list, GetPlantPredictedPowerMidTermOutput.class);
        ThirdActivePowerServiceHelper.getActivePower(context, outputs, input.getPlantId(), showTime);
        return XListResult.success(outputs);
    }

    private QueryWrapper<PlantPredictedPowerShortTermEnt> setQueryWrapper(String plantId, String startTime, Integer days) {
        Date date = new Date();
        QueryWrapper<PlantPredictedPowerShortTermEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(PlantPredictedPowerShortTermEnt::getPlantId, plantId);
        if (XStringUtils.isEmpty(startTime)) {
            queryWrapper.lambda().ge(PlantPredictedPowerShortTermEnt::getDataDate, date);
            queryWrapper.lambda().le(PlantPredictedPowerShortTermEnt::getDataDate, XDateUtils.addDays(date, days));
        } else {
            queryWrapper.lambda().ge(PlantPredictedPowerShortTermEnt::getDataDate, startTime);
            queryWrapper.lambda().le(PlantPredictedPowerShortTermEnt::getDataDate, XDateUtils.addDays(XDateUtils.parse(startTime), days));
        }
        queryWrapper.lambda().orderByAsc(PlantPredictedPowerShortTermEnt::getDataDate);
        return queryWrapper;
    }
}